% dpsolver  Solves discrete-state/action inf horizon dynamic program
% by policy iteration
% Call: [v,x,pstar] = dpsolver(reward,transitions,delta,pflag,jflag);
% INPUTS
% The possible fields of the model structure variable are:
% delta : discount factor, a scalar on (0,1]
% reward : nxm matrix of reward values, n number of states, m number
% of controls
% transitions : for deterministic problems an nxm matrix of values on {1,...,m}
% and for stochastic problems  mxnxn a set of m nxn probability matrices, i.e.,
% for each action what is the probability of moving from one state to another.
% jflag : set to one if Jacobi modification (optional) 
% pflag : set to one if policy iteration and zero for value iteration (optional)
% Tol : numerical tolerance for value iteration
% preset is policy iteration without any modifications
% (may improve convergence for highly statonary systems)
% OUTPUTS
% v       : value function (nx1 or nxT+1)
% x       : optimal controls (nx1 or nxT)
% pstar   : optimal transition probability matrix (nxn or nxnxT+1)

function [v,x,pstar] = dpsolver(reward,transitions,delta,jflag,pflag,Tol)
maxit=100;
if length(size(transitions))<3
 [n,m] = size(transitions);
 P = sparse(1:n*m,transitions(:),1,n*m,n);
else
 P = transitions;
 [n,m] = size(reward);
 P = permute(P,[2 1 3]);
 P = reshape(P,m*n,n);
end
if nargin==4 && jflag==1
PS=reshape(diag(repmat(P',[m 1])),[n m]); %The probabilities of staying in the current state
dfac=1-PS.*delta;
else
jflag=0;
end

if nargin<5
pflag=1;
end

if nargin<6
Tol=100*eps;
end

[n,m] = size(reward);
v=zeros(n,1);
%Policy iteration
x=zeros(n,1);
for it=1:maxit
 vold = v;
 xold=x;
 if jflag==0
  [v,x]=max(reward+reshape(delta.*(P*v),n,m),[],2);
 else
  [v,x]=max((reward+reshape(delta.*(P*v),n,m)-PS.*repmat(v,[1 m]))./dfac,[],2);
 end
 if pflag==1
 ind=n*x+(1-n:0)';
 fstar = reward(ind);
 pstar = P(ind,:);
 v = (speye(n)-delta.*pstar)\fstar;
 end
 if pflag==1 && all(x==xold)
	break; 
 elseif norm(vold-v)<Tol
	ind=n*x+(1-n:0)';
	pstar = P(ind,:);
	break;
 end
end
