% PATHSOLVER Octave function for finding the elementary subpaths of two player supergames.
% Inputs: 
% game = multidimensional table of players' stage game payoffs 
% game(i,j,k)=player k's payoff when player 1 takes action i and player two takes action j
% vmini 2 x 1 vector where vmini(i) the smallest payoff for player i (can be left empty)
% delta = vector of discount factors, delta(i) = player i's discount factor
% maxcount = maximum number of iteration steps (optional, preset=20)
% Tol = tolerance for the numerical error (optional, preset = eps) 
% cflag = set 1 if you want to carry out tests for infinitely long subpaths for equal discount
% factors (preset = 1)
% Outputs:
% E cell array of elementary supbaths of different length (E{i} paths of length i)
% WS working set after maxcount steps unless algorithm has terminated and WS is empty
% Einf infinitely long elementary subpaths (outputted only in the case of equal discount factors)
% The last number of a path in Einf indicates the action profile that is repeated infinitely.
% U utilities in a table U(i,k) player i's stage game payoff corresponding to k'th action profile
% labeling of action profiles corresponds to second index of U, e.g., a paths 123, refers to 
% path in which the first three payoff vectors are U(:,1), U(:,2), and U(:,3)
% outgraph (optional): cell-array for the graph corresponding to the elementary set;
% outgraph{1} are the labels of nodes, outgraph{2} is the adjacency matrix
% Requirements:
% gamepreprocess.m function for preprocessing game table
% fastsetdiff.m auxiliary function for carrying out set difference
% fastintersect.m set intersection
% grahamscan.m function for computing the convex hull in two dimensions (slightly tuned Graham-Scan)
% inhulltest.m function for testing whether a vector belongs to a convex hull of a given vertex set
% paths2tree.m function that turns a set of subpaths into a tree.
% tree2graph.m function that turns a tree two a graph
% minpayoffs.m function for computing the minimum payoffs that can follow the action profiles, this 
% requires dpsolver.m (dynamic programming)
% postprocess.m function for post processing the output into the final form
% path2nodes.m turns a path of length n with a given base number into a set of sequences of indices.
% matrix of the graph.
% auxflipper.m flipping paths around
% appendgraph.m appending a graph with infinitely long paths
function [E,WS,Einf,U,outgraph]=pathsolver(game,vmini,delta,maxcount,Tol,cflag)

[U,D,devs,conts,vmin,vmax]=gamepreprocess(game,vmini,delta);%Preprocessing

%U utilities in a table U(i,k) player i's stage game payoff corresponding to k'th action profile
%D is a table corresponding to U where the payoffs are the RHS values of IC conditions
%devs=players deviation payoffs (dimensions as in U)
%conts=players required continuation payoffs (dimensions as in U)
%vmin = smallest payoffs (equal to vmini if vmini is given)
%vmax = largest stage game payoffs
%NOTE:  U becomes empty for vmin too large

if ~isempty(U)
U1=U(1,:);
U2=U(2,:);
dev1=D(1,:);
dev2=D(2,:);
C1=conts(1,:);
C2=conts(2,:);

%Parameters
base=10^ceil(length(U1)/10);	
vmax=[max(U(1,:)) max(U(2,:))];
if nargin<4
maxcount=20;
end
if nargin<5
Tol=eps;
end
if nargin<6
cflag=1;
end


%Step 1
E{1}=find(C1<=vmin(1) & C2<=vmin(2));
Con1e{1}=C1(E{1});
Con2e{1}=C2(E{1});
aux=find(C1<= vmax(1) & C2<= vmax(2));
WS{1}=fastsetdiff(aux,E{1}); %Star set, Working set
Con1{1}=C1(WS{1});
Con2{1}=C2(WS{1});
Einf{1}=[];
if cflag==1 && delta(1)==delta(2)
verts=grahamscan(U,eps);
end

cntr=2;
%Step 2
while cntr<=maxcount
Con1p=[];
Con2p=[];
Con1pp=[];
Con2pp=[];
labelp=[];
testlabels=[];

for i=1:length(U1)
preaux1=(Con1{cntr-1}-(1-delta(1))*U1(i))./delta(1);
preaux2=(Con2{cntr-1}-(1-delta(2))*U2(i))./delta(2);
aux1=max(preaux1,vmin(1));
aux2=max(preaux2,vmin(2));
auxlabels=i*base^(cntr-1)+WS{cntr-1};

newelems=find(aux1<=C1(i) & aux2<=C2(i));%Potential new elementary set

if cflag==1 && delta(1)==delta(2) %Those subpaths for which cons stay in FIR
newelems=inhullfinder(newelems,aux1,aux2,verts,eps);
end
labelp=[labelp auxlabels(newelems)];
Con1pp=[Con1pp aux1(newelems)];%These are saved only for diagnostic purposes
Con2pp=[Con2pp aux2(newelems)];

auxi=find(aux1<= vmax(1) & aux2<= vmax(2)); %these are the potential working set elements to be added
indices=fastsetdiff(auxi,newelems);

if cflag==1 && delta(1)==delta(2) %Those subpaths for which cons stay in FIR
indices=inhullfinder(indices,aux1,aux2,verts,eps);
end

testlabels=[testlabels auxlabels(indices)]; %These are the potential additional working set elements
Con1p=[Con1p preaux1(indices)];
Con2p=[Con2p preaux2(indices)];
end

if ~isempty(labelp) %Update the elementary set if new potential subpaths are available 
[E{cntr},Con1e{cntr},Con2e{cntr}]=condition8(labelp,E,WS,base,Con1pp,Con2pp,cntr);

else
E{cntr}=[];
Con1e{cntr}=[];
Con2e{cntr}=[];
end

if ~isempty(testlabels) %Update the working set
[WS{cntr},Con1{cntr},Con2{cntr}]=condition8(testlabels,E,WS,base,Con1p,Con2p,cntr);
else
WS{cntr}=[];
Con1{cntr}=[];
Con2{cntr}=[];
end

if cflag==1 && delta(1)==delta(2) && ~isempty(WS{cntr}) %Additional steps for equal discount factors
if cntr==2
[inflabels,infconts]=findinfverts(E{2},U1,U2,verts,base);
elseif cntr>2 && length(inflabels)>=1
[WS{cntr},Con1{cntr},Con2{cntr},Einf{cntr}]=step2c(inflabels,infconts,WS{cntr},Con1{cntr},Con2{cntr},vmin,base,cntr,Tol);
[WS{cntr},Con1{cntr},Con2{cntr},outwinf]=step2d(WS,Con1,Con2,base,vmin,Tol);
Einf{cntr}=[Einf{cntr} outwinf];
else
Einf{cntr}=[];
Con1einf{cntr}=[];
Con2einf{cntr}=[];
end
end

if isempty(WS{cntr}) 
break; %Termination if working set is empty
end

cntr=cntr+1;
end %Step 2 ends here

%Step 3, removing non-equilibrium parts and forming of graph
%Turn paths into a tree
[nodeset,A,pathflags]=paths2tree(E,base);
if ~isempty(nodeset)
%Turn tree into a graph
[Aout,L,nodeset]=tree2graph(nodeset,A,base); %note some subpaths may get removed here
if cflag==1 && delta(1)==delta(2) && ~isempty(L)%In this case the graph is appended with infinitely repeated action profiles
[Aout,L]=appendgraph(Aout,L,Einf,base);%Append the graph with Inf paths
end
else
L=[];
end

if ~isempty(L)
%Post processing (Step 4)
%Finding the smallest payoffs that can follow playable action profiles
[vmina,La]=minpayoffs(Aout,L,U,delta);
%Removing non-equilibrium parts
[E,vout,nodesds,vouds]=postprocess(nodeset,U,vmina,La,delta,base);
if ~isempty(Einf)
for i=1:length(Einf)
Einf{i}=auxflipper(Einf{i},base);%Flipping around paths in Einf to get correct output
end
end

if nargout==5 
outgraph{1}=L;
outgraph{2}=Aout;
end
else
disp('Algorithm terminated with no paths.')
E={};
Einf={};
outgraph={};
end

else
disp('Either vmin is too large or there are no SPE paths.')
E={};
WS={};
Einf={};
outgraph={};
end

endfunction
%-------------The main caller ends here----------------------------------

function [output,outc1,outc2]=condition8(labels,E,WS,base,Con1p,Con2p,cntr)
%Function for testing condition (8)
N=length(WS);

if ~isempty(labels)
label=labels;
if nargin>=5
outc1=Con1p;
outc2=Con2p;
else
outc1=[];
outc2=[];
end

for cnt=1:N

trials=floor(label./base^cnt);
indic=(trials==repmat(WS{N-cnt+1}',[1 length(label)]));
ind1=(sum(indic,1)>0); %The test for star set
if ~all(ind1)
ind2aux=phattest(trials(~ind1),E,base);

ind2=false(size(ind1));
ind2(~ind1)=ind2aux;
indicator=(ind1 | ind2);
else
indicator=ind1;
end

if ~isempty(indicator) %continue with the labels that are possible
label=label(indicator);
if nargin>=5
outc1=outc1(indicator);
outc2=outc2(indicator);
end
else %nothing left
output=[];
break
end
if isempty(label)
break
end
end
output=label;
else
output=[];
outc1=[];
outc2=[];
end
endfunction
%---------------------------------------------------------------------

function [inflabels,infconts]=findinfverts(E2,U1,U2,verts,base);
%Finding payoffs such that their infinite repetition yields a vertex in verts.
%Outputs are the label of such action profiles and the corresponding payoffs.
secs=floor(E2./base);
firsts=E2-secs*base;
trials=firsts(find(secs==firsts)); %indices of interesting action profiles
if ~isempty(trials)
indic1=(U1(trials)'==repmat(verts(1,:),[length(trials) 1]));
indic2=(U2(trials)'==repmat(verts(2,:),[length(trials) 1]));
inflabels=find(sum(indic1 & indic2,2)>0); %indices of vertex payoffs
infconts=[U1(:,inflabels)';U2(:,inflabels)'];
else
inflabels=[];
infconts=[];
end
endfunction

%-----------------------------------------------------------------------------------
function [outws,outcon1,outcon2,outwinf]=step2c(inflabels,infconts,WS,Con1,Con2,vmin,base,ctr,Tol)
%Testing which continuation values correspond to those payoff vertices in which an 
%action profile is repeated inifnitely many times.
remainder=[1:1:length(WS)];%This is the set of indices of paths in the working set.
outwinf=[];
aux1=max(Con1,vmin(1));
aux2=max(Con2,vmin(2));
for j=1:length(infconts(1,:))
%The set of indices of paths such that some vertex produces the required continuation 
infidx=find(abs(aux1(remainder)-infconts(1,j))<Tol & abs(aux2(remainder)-infconts(2,j))<Tol);
if ~isempty(infidx)
outwinf=[outwinf j*base^(ctr)+WS(infidx)];
remainder=fastsetdiff(remainder,infidx);
end
if isempty(remainder)
break
end
end

if ~isempty(remainder)
outws=WS(remainder);%update the working set
outcon1=Con1(remainder); %update the Cons corresponding to the working set
outcon2=Con2(remainder);
else
outws=[];
outcon1=[];
outcon2=[];
end
endfunction

%---------------------------------------------------------------
function outelems=inhullfinder(inelems,aux1,aux2,verts,Tol);
%Testing whether vectors [aux1(inelems(i)) aux2(inelems(i))] are in the hull of verts.
inhullflag=false(size(inelems));
for j=1:length(inelems)
inhullflag(j)=inhulltest([aux1(inelems(j)) aux2(inelems(j))]',verts,Tol);
end
outelems=inelems(inhullflag);
endfunction

%---------------------------------------------------------------------
function [outws,outcon1,outcon2,outwinf]=step2d(WS,Con1,Con2,base,vmin,Tol)
%This is for updating the infinitely long elementary subpaths and the working set.
%If we have found subpaths that rely on infinite repetition of an action profile include them
%in outwinf and remove from the working set.

cnt=length(WS);

if length(WS{cnt})==length(WS{cnt-1})
aux11=max(Con1{cnt},vmin(1));
aux21=max(Con2{cnt},vmin(2));
aux12=max(Con1{cnt-1},vmin(1));
aux22=max(Con2{cnt-1},vmin(2));

%Find elements from the working set such that the beginning of the path has remained exactly the same and the continuation has not changed
indices=find(WS{cnt}-floor(WS{cnt}./base^(cnt-1)).*base^(cnt-1)==WS{cnt-1} & abs(aux11-aux12)<Tol & abs(aux21-aux22)<Tol);
if ~isempty(indices)
outwinf=WS{cnt}(indices);
remainder=fastsetdiff([1:1:length(WS{cnt})],indices);
outws=WS{cnt}(remainder);
outcon1=Con1{cnt}(remainder);
outcon2=Con2{cnt}(remainder);
else
outwinf=[];
outws=WS{cnt};
outcon1=Con1{cnt};
outcon2=Con2{cnt};
end
else
outwinf=[];
outws=WS{cnt};
outcon1=Con1{cnt};
outcon2=Con2{cnt};
end
endfunction

%-------------------------------------------------------------------------------
function ind=phattest(trials,E,base)
%Testing which of the trials (list of subpaths) have a fragment in E.
%Output is a vector of indicators (true if found) 

N=length(E);
M=length(trials);
if size(trials,2)>size(trials,1)
trials=trials';
end
qjmat=trials;
if N>=2
qjmat=[mod(trials,base.^[1:N-1]) trials];
end
for cnt=1:M
ind(cnt)=(sum(sum(qjmat(cnt,:)'==repmat(cell2mat(E(1:N)),[size(qjmat,2) 1])))>0);
end
endfunction

