function [Aout,L,outnodeset]=tree2graph(nodeset,A,base,maxcount)
%Turning a tree into a graph.
%Inputs:
%the set of subpaths stored in a row vector nodeset
%A = adjacency matrix of the tree
%base = base number for storing subpaths
%maxcount = maximum path length (optional)
%Outputs:
%Aout = adjacency matric of the graph, root node is the  first one
%L = labels of action profiles played in each node
%outnodeset = subpaths after removing non-equilibrium parts

if nargin<4
maxcount=Inf;
end

Aout=zeros(size(A)+1);
L=zeros(length(A)+1,1);
c2root=find(floor(nodeset./base)==0); %these are connected to the root node
outnodeset={};

leafnodes=find(sum(A,2)==0);
innernodes=fastsetdiff([1:1:length(nodeset)],leafnodes);

if ~isempty(innernodes)
Aout(1+innernodes,:)=[zeros(length(innernodes),1) A(innernodes,:)];
for i=1:length(innernodes)
L(1+innernodes(i))=nodeset(innernodes(i))-floor(nodeset(innernodes(i))/base)*base;
end
end

trial=fastintersect(leafnodes,c2root); %leafnodes that are root nodes at the same time

if ~isempty(trial)
Aout(1+trial,1+c2root)=1;
L(trial+1)=nodeset(trial);
outnodeset(trial+1)=num2cell(nodeset(trial));
end

workset=fastsetdiff(leafnodes,trial); %these are the leafnodes that are not connected to the root
searchnodeset=nodeset;
cnt=2;

while ~isempty(workset) && cnt<=maxcount
trialnodes=find(floor(nodeset(workset)./base^(cnt))==0 & floor(nodeset(workset)./base^(cnt-1))>0); %go count by count
trialnodes=workset(trialnodes);
workset=fastsetdiff(workset,trialnodes);

for i=1:length(trialnodes)
[frag,Laux]=findfragintree(nodeset(trialnodes(i)),searchnodeset,Aout,base);%finding the fragment in the tree
if ~isempty(frag) && frag>0
pathm=floor(nodeset(trialnodes(i))/base);
indfrom=find(nodeset==pathm);
indout=find(nodeset==frag);
Aout(:,trialnodes(i)+1)=0;
Aout(indfrom+1,indout+1)=1;
auxind=find(searchnodeset==nodeset(trialnodes(i)));
searchnodeset(auxind)=-Inf;

L(trialnodes(i)+1)=Laux;

if length(outnodeset)<indfrom+1 || ~isempty(outnodeset{indfrom+1})
outnodeset{indfrom+1}=[outnodeset{indfrom+1} nodeset(trialnodes(i))];% nodes that are to be kept
else
outnodeset{indfrom+1}=nodeset(trialnodes(i));
end
end%end of if
end%end of for
cnt=cnt+1;
end%end of while
Aout(1,c2root+1)=1;
if ~isempty(outnodeset)
[Aout,L,outnodeset]=reducemat(Aout,L,outnodeset);
outnodeset=cell2nodeset(outnodeset);
else
L=[];
end
endfunction

%------------------------------------------------------------------------------------
function [Aout,Lout,outnodeset]=reducemat(A,L,nodeset)
%Removing non-equilibrium parts.
stopflag=false;
Aout=A;
Lout=L;
outnodeset=nodeset;

while stopflag==false
 indic=find(sum(Aout,2)>0);
if length(indic)==length(Aout)
stopflag=true;
else
outnodeset=outnodeset(indic);%the first index is one, which corresponds to the root node, in nodeset the first index is one of the nodes
Aout=Aout(indic,indic);
Lout=Lout(indic);
end
end
endfunction

%----------------------------------------------------------------------------------------
function outnodeset=cell2nodeset(nodeset);
outnodeset=[];
for i=1:length(nodeset)
outnodeset=[outnodeset nodeset{i}];
end
endfunction

%----------------------------------------------------------------------------------------
function [frag,L]=findfragintree(trial,nodeset,A,base)
%Finding the relevant fragment of trial in the tree corersponding to adjacency matrix A.
%Trial is the subpath that we are interested in, the ordering of actions in trial is reversed, e.g.,
%sequence 1234 (base=10) means that we first play 4 then 3 and so on.
%nodeset is the set of nodes (sequences of action profiles) corresponding to the nodes of the tree
%base is the base number for indexing subpaths.

frag=[];
L=0;
n=numel(num2str(trial));
qjmat=path2nodes(trial,n,base);
qjmat=qjmat(1:end-1);%the final element of the path itself which we are not interested in

if ~isempty(qjmat)
qjmat=auxflipper(qjmat,base);
idx=max(find(sum(qjmat'==nodeset,2)));

%Next there should be a test whether we have an inner node
if ~isempty(idx)
trialm=qjmat(idx);%this is the trial node unless it is a leaf node, in which case use its predecesssor
idx=find(nodeset==trialm);

predecessors=find(A(:,idx+1)>0);
successors=find(A(idx+1,:)>0);

if (~isempty(predecessors) & ~isempty(successors))
frag=trialm;
L=mod(frag,base);
else
frag=floor(trialm/base);
L=mod(frag,base);
end
end
end
endfunction
